GLREF_C NOTE *ePointToNote(INT pos);
GLREF_C INT eAppendNullEntry(VOID);
GLREF_C VOID eZeroNote(NOTE *pn);
GLREF_C INT eDeleteNote(INT pos);
GLREF_C VOID eInitialiseEngine(VOID);
GLREF_C INT eCountTaggedNotes(VOID);
GLREF_C VOID eTagAllNotes(INT clear);
GLREF_C VOID eTagCurrentNote(INT clear);
GLREF_C VOID eTakeCopy(NOTE *targ,NOTE *src);
GLREF_C VOID eMoveNoteTo(INT newpos,INT pos);

GLREF_C VOID InitLinkServer(VOID);
